--[[
	CS2D Script made by Bowlinghead.
	Part of the "Buy Request" Script to emulate Valorants digital drop request system in CS2D
	Uploaded on UnrealSoftware.de
]]


buyReq = {}


-- Can the same player request more than 1 weapon? (Note: max 9 total requests at any given time, so 1 player can spam it :( )
-- true = multi drop
-- false = only 1 drop per player
isMultiRequest = false

-- Can a TT request a M4A1? (Or CT a AK47)
-- true = no, a TT cant get a M4A1 (any M4A1 order will then be swapped with an AK47 order)
-- false= yes, a TT can get a M4A1 
-- note: Ofc you would need team-sens weapon in the first place inside your mp_buymenu
-- So when you would only include AK47 and not M4A1 in your mp_buymenu, then CTs would still be able to buy M4A1s with this setting enabled
isTeamSensitveWeaponRequests = true




-- Will bots buy guns for you (when they have enough money, etc)
-- true = bots do buy guns for humans
-- false = bots will ignore you
isBotAnsweringRequests = false

-- generate CT/TT table
function getTeamWeaponsTable()
	local result = {}
	result[21] = 23;
	result[32] = 30;
	result[33] = 31;
	result[39] = 38;
	return result;
end


--[[

	TRACKER - tracks if buytime is currently over or not.
	
]]--
tracker = { };
tracker.isBuyTime = false


function tracker.onStartRound()
	tracker.isBuyTime = true;
	timer(game("mp_buytime")*60000,"tracker.onEndBuyPhase") -- cs2d:timer uses ms, while mp_buytime uses minutes
end

function buyReq.onStartRound()
	-- reset everything
	tracker.onStartRound();
	
	for _, id in pairs(player(0,"tableliving")) do
		playerCurrentGroup[id] = -1;
		playerConfig[id] = {}
	end
	
	-- empty drop tables
	dropsCurrentAskedCT = {}
	dropsCurrentAskedTT = {};
end

function tracker.onEndBuyPhase()
	tracker.isBuyTime = false;
	for _, id in pairs(player(0,"tableliving")) do
		playerCurrentGroup[id] = -1;
	end
	dropsCurrentAsked = {}
end


--[[
	HELPER FUNCTIONS
]]

function isInBuyzone(id)
	-- can be commented out (?), see below or forum upload thread
	local spawnList = {}
	local spawnId, dx, dy
	
	-- team depending
	if (player(id,"team") == 1) then
		--https://www.cs2d.com/entities.php?cat=all&entity=Info_CT#entity
		spawnId = 0;
	else
		spawnId = 1;
	end
	
	-- get all spawns of team
	for _,e in pairs(entitylist()) do
		if (entity(e.x, e.y, "type") == spawnId) then
			dx = e.x - player(id,"tilex")
			dy = e.y - player(id,"tiley")
			-- is player inside spawnzone?
			if (math.abs(dx) <= 2) then
				if (math.abs(dy) <= 2) then
					return true;
				end
			end
		end
	end
	return false
end


--[[ 
	BUY MENU
]]

local gameBuyMenu = game("mp_buymenu")
--print(gameBuyMenu)
if (gameBuyMenu == "") then -- no custom buy menu -> use default
	gameBuyMenu = "1,2,3,4,5,6|10,11|20,21,22,23,24|30,31,32,33,34,35,36,37,38,39|40|51,52,53,54,57,58,59"
	print("No custom mp_buymenu found! Using default string...")
else
	print("Found mp_buymenu!")
end
print("*** Using this: ***")
print(gameBuyMenu)
print("as mp_buymenu string")

-- the buymenu as a table full of #group lists with item-ids
buyMenu = getBuyMenu(gameBuyMenu)
-- the buymenu as a table full of strings for cs2d:menu()
buyMenuString = getBuyMenuString(buyMenu);


if (isTeamSensitveWeaponRequests) then
	buyReq.teamWeapons = getTeamWeaponsTable()
end


--[[

	HOOKS
	
]]

local defaultPrio = 0;


function init()
	addhook("serveraction","buyReq.serveraction",defaultPrio)
	addhook("startround","buyReq.onStartRound",defaultPrio)
	addhook("menu", "buyReq.menu", defaultPrio)
	
	parse("endround")
end


--[[
	MENU
]]
local menuButtonAsk = 1;
local menuButtonGive = 2;

--[[ playerCurrentGroup
	represents the current menu the player is currently in.
	-2 = is in "Drops Menu"
	-1 = not in any of my menus (or any menu at all)
	1-9 = current selected gungroup (1 = Handgun,..., 3 = Sub Machine Gun, ..., 6 = Equipment)
]]--
playerCurrentGroup = {}

playerConfig = {}
dropsCurrentAskedCT = {}
dropsCurrentAskedTT = {}

function generateDropMenu(id)
	playerConfig[id] = generateDropConfig(id);
	showDropMenu(id, playerConfig[id]);
end


--[[ dropConfig <=> {
	[1] = link1
	[2] = link2
	... 
	[9] = link9 
	(max 9!)
}

a link is just a table = { id, wpn } (id is the target that gets the drop of wpn)

]]--

function showDropMenu(id, dropConfig)
	local result = "Drops Menu@b";
	local dropsCurrentAsked = {}
	local nextResultRow;
	
	
	-- determine team
	if (player(id,"team") == 1) then
		dropsCurrentAsked = dropsCurrentAskedTT;
	else
		dropsCurrentAsked = dropsCurrentAskedCT;
	end
	
	for _,link in ipairs(dropConfig) do
		nextResultRow = itemtype(link[2],"name").." ("..itemtype(link[2],"price").."$) |"..player(link[1],"name")
		
		-- not enough money -> make purchase unavailable
		if (player(id,"money") < itemtype(link[2],"price")) then
			nextResultRow = "("..nextResultRow..")";
		end
		
		result = result..","..nextResultRow
	end
	
	menu(id, result);

end


function generateDropConfig(id)
	local result = {}
	local dropsCurrentAsked = {}
	
	-- determine team
	if (player(id,"team") == 1) then
		dropsCurrentAsked = dropsCurrentAskedTT;
	else
		dropsCurrentAsked = dropsCurrentAskedCT;
	end

	for k,link in pairs(dropsCurrentAsked) do
		-- Errors (cant menu more than 9 dishes for e.g.)
		if (k > 9) then
			break;
		end
		
		-- dont drop for yourself!
		if (link[1] ~= id) then
			result[#result+1] = link
		end
	end
	
	
	return result;
end


function dropRequest(id, wpn)
	local dropsCurrentAsked = {}

	-- determine team
	if (player(id,"team") == 1) then
		dropsCurrentAsked = dropsCurrentAskedTT;
	else
		dropsCurrentAsked = dropsCurrentAskedCT;
	end
	
	-- Abbruchbedingungen
	-- Cant ask for primary/secondary when already owning a primary/secondary
	if (wpn >= 10) then
		for _,wpnOwned in ipairs(playerweapons(id)) do
			if (wpnOwned >= 10 and wpnOwned < 50) then
				msg2(id, "You can´t request a primary drop; you already own a primary weapon ("..itemtype(wpnOwned,"name")..")!")
				return;
			end
		end
	else
		for _,wpnOwned in ipairs(playerweapons(id)) do
			if (wpnOwned < 10) then
				msg2(id, "You can´t request a secondary drop; you already own a secondary gun ("..itemtype(wpnOwned,"name")..")!")
				return;
			end
		end
	end
	
	-- Multi Request
	if (isMultiRequest) then
		-- Cant ask for same weapon twice
		for _, link in ipairs(dropsCurrentAsked) do
			if (link[2] == wpn) then
				msg2(id, "You can´t request the same drop twice!")
				return;
			end
		end
	else
		-- Did this player already request a drop?
		for linkId, link in ipairs(dropsCurrentAsked) do
			if (link[1] == id) then
				-- kill old request
				dropsCurrentAsked[linkId] = nil;
				break; -- if isMultiRequest==false, then there cant be more than 1 request.
			end
		end
	end
	
	-- Team Sensitve Drop Requests
	if (isTeamSensitveWeaponRequests) then
		
		if (player(id,"team") == 1) then
			-- If requested wpn is a CT only gun
			if (buyReq.teamWeapons[wpn] ~= nil) then
				-- reformulate request to corresponding weapon (M4 request -> AK request)
				wpn = buyReq.teamWeapons[wpn];
			end
		else
			for k, v in pairs(buyReq.teamWeapons) do
				-- if requested wpn is a TT only 
				if (wpn == v) then
					wpn = k; -- reformulate
					break;
				end				
			
			end
		end
	end
	
	local newLink = {}
	newLink[1] = id;
	newLink[2] = wpn;
	table.insert(dropsCurrentAsked, newLink);
	msg2(id, "You requested a drop ("..itemtype(wpn,"name")..")")
	
	-- Let the bots buy you the guns first!
	if (isBotAnsweringRequests) then
		dropBotWillAnswer(newLink);
	end
	
	dropListUpdate();
end


function dropBotWillAnswer(link) 
	local botList = {}
	
	for _, v in ipairs(player(0,"tableliving")) do
		if (player(v,"bot")) then
			if (player(v,"health") > 0) then
				
				-- bot has enough money
				if (player(v,"money") >= itemtype(link[2],"price")) then
					dropExecute(v, link); -- lets hope the bot is in a buyzone :)
					return;
				end
			
			end
		end
	end
end


-- Will delete the drop request list entry and refresh list and drop the weapon (id = the guy who looses money; target id is in link[2])
function dropExecute(id, link)
	local dropsCurrentAsked = nil

	-- determine team
	if (player(id,"team") == 1) then
		dropsCurrentAsked = dropsCurrentAskedTT;
	else
		dropsCurrentAsked = dropsCurrentAskedCT;
	end
	
	-- Abbruchbedingungen: todo target dead, etc.? not neccessary i guess
	-- Target already owns primary/secondary gun
	if (link[2] >= 10) then
		for _,wpnOwned in ipairs(playerweapons(link[1])) do
			if (wpnOwned ~= 50) then
				if (wpnOwned >= 10) then
					msg2(id, "The player that requested a drop already owns a primary weapon! Drop denied.")
					-- delete entry
					dropListDelete(dropsCurrentAsked, link)
					dropListUpdate()
					return;
				end
			end
		end
	else
		for _,wpnOwned in ipairs(playerweapons(link[1])) do
			if (wpnOwned < 10) then
				msg2(id, "The player that requested a drop already owns a secondary gun! Drop denied.")
				-- delete entry
				dropListDelete(dropsCurrentAsked, link)
				dropListUpdate()
				return;
			end
		end
	end
	
	-- --[[ -- YOU MIGHT WANT TO COMMENT OUT THE NEXT 3 LINES;
	-- most likely your mp_freeztime matches roughly your mp_buytime, making this expensive check redundant
	-- most likely the gameplay doesnt alter when commenting this out, but you safe perfomance
	-- buyer and target have to be in buyzone!
	if (isInBuyzone(id) == false or isInBuyzone(link[1]) == false) then
		msg2(id, "You can´t drop; At least one of you is not in a buyzone!")
		return;
	end
	--]]--
	-- the swap
	parse("setmoney "..id.." "..player(id,"money")-itemtype(link[2],"price"))
	parse("equip "..link[1].." "..link[2]);
	msg2(link[1], "You received your requested drop ("..itemtype(link[2],"name")..") from "..player(id,"name"))
	
	-- delete entry
	dropListDelete(dropsCurrentAsked, link)
	dropListUpdate()
	
end

function dropListDelete(dropsCurrentAsked, link)
	-- delete entry
	for k, currentLink in pairs(dropsCurrentAsked) do
		if (link == currentLink) then
			dropsCurrentAsked[k] = nil
		end
	end

end


function dropListUpdate()
	-- refresh all other´s drop menu! (we updated the Drop Menu, everyone wants to see it :o)
	for idOther, v in ipairs(playerCurrentGroup) do
		if (v == -2) then
			generateDropMenu(idOther)
		end
	end
end

function buyReq.serveraction(id,b)
	if (tracker.isBuyTime ~= true) then
		print("BUYTIME PASSED")
		return;
	end
	
	if (player(id,"health") > 0) then
		if (b==menuButtonAsk) then
			playerCurrentGroup[id] = 0
			menu(id, buyMenuString[0])
		else
			playerCurrentGroup[id] = -1;
		end
		
		if (b==menuButtonGive) then
			playerCurrentGroup[id] = -2;
			generateDropMenu(id);
		end
	end
end


function buyReq.menu(id,t,b)
	if (b==0) then
		playerCurrentGroup[id] = -1;
		return;
	end

	if (t=="Ask For Drop") then
		playerCurrentGroup[id] = b;
		menu(id, buyMenuString[b])
		return;
	end
	
	if (playerCurrentGroup[id] > 0) then
		-- In here: Player is in a group menu for specific guns -> every btn-press 1-9 will request a drop!
		local wpn = buyMenu[playerCurrentGroup[id]]
		dropRequest(id, wpn[b]);
	end
	
	-- TODO hiers das problem. Was hat der spieler fürn menu angezeigt bekommen? Droplist noch aktuell? Welcher Button saght was aus???
	if (t=="Drops Menu") then
		dropExecute(id, playerConfig[id][b])
	end
end

-- Scripteinstiegspunkt
init()